/*************************************************************************
 *                                                                       *
 * Vega FEM Simulation Library Version 4.0                               *
 *                                                                       *
 * "basicAlgorithms" library , Copyright (C) 2018 USC                    *
 * All rights reserved.                                                  *
 *                                                                       *
 * Code authors: Yijing Li, Jernej Barbic                                *
 * http://www.jernejbarbic.com/vega                                      *
 *                                                                       *
 * Research: Jernej Barbic, Hongyi Xu, Yijing Li,                        *
 *           Danyong Zhao, Bohan Wang,                                   *
 *           Fun Shing Sin, Daniel Schroeder,                            *
 *           Doug L. James, Jovan Popovic                                *
 *                                                                       *
 * Funding: National Science Foundation, Link Foundation,                *
 *          Singapore-MIT GAMBIT Game Lab,                               *
 *          Zumberge Research and Innovation Fund at USC,                *
 *          Sloan Foundation, Okawa Foundation,                          *
 *          USC Annenberg Foundation                                     *
 *                                                                       *
 * This library is free software; you can redistribute it and/or         *
 * modify it under the terms of the BSD-style license that is            *
 * included with this library in the file LICENSE.txt                    *
 *                                                                       *
 * This library is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file     *
 * LICENSE.TXT for more details.                                         *
 *                                                                       *
 *************************************************************************/

#ifndef FILEIO_H
#define FILEIO_H

#include <fstream>
#include <functional>
#include <string>

// read each line from fin
// empty line will be skipped, so input parameter line to processLine is always not empty
// the input function processLine should return 0 when success
// return 0 when success, return the error code of processLine otherwise
// Note: when using filename as input parameter, the function returns 1 when it fails to open the file
int readEachLine(const std::string & filename, std::function<int(std::string & line)> processLine, const char * comment = "#");
int readEachLine(std::istream & fin, std::function<int(std::string & line)> processLine, const char * comment = "#");

// get the directory part of a path
// examples: a/b/c -> a/b     a -> .
std::string getPathDirectoryName(const std::string & path);
#endif
